// FIRMWARE VERSION     1.1
#include QMK_KEYBOARD_H
#include "app_ble_func.h"

#ifdef SSD1306OLED
  #include "ssd1306.h"
#endif

#include "eeprom.h"

// define variables for reactive RGB
bool TOG_STATUS = false;
int RGB_current_mode;
#ifdef RGBLIGHT_ENABLE
//Following line allows macro to read current RGB settings
extern rgblight_config_t rgblight_config;
#endif

const uint8_t is_master = IS_LEFT_HAND;

void nrfmicro_power_enable(bool enable);
extern uint8_t nrfmicro_switch_pin(void);

bool has_usb(void);

// Each layer gets a name for readability, which is then used in the keymap matrix below.
// The underscores don't mean anything - you can have a layer called STUFF or any other name.
// Layer names don't all need to be of the same length, obviously, and you can also skip them
// entirely and just use numbers.

enum layer_number {
  _QWERTY = 0, 
  _LOWER = 1, 
  _RAISE = 2, 
  _ADJUST = 3, 
  _4 = 4,
  _5 = 5,
  _6 = 6,
  _7 = 7,
  _8 = 8,
  _9 = 9,
  _10 = 10,
  _11 = 11,
  _12 = 12,
  _13 = 13,
  _14 = 14,
};

enum custom_keycodes {
    AD_WO_L = SAFE_RANGE, /* Start advertising without whitelist        */
    BLE_DIS,              /* Disable BLE HID sending                    */
    BLE_EN,               /* Enable BLE HID sending                     */
    USB_DIS,              /* Disable USB HID sending                    */
    USB_EN,               /* Enable USB HID sending                     */
    BLE_SEL,              /* Enable BLE Only                            */
    USB_SEL,              /* Enable USB Only                            */
    DELBNDS,              /* Delete all bonding, all Bluetooth Devices  */
    ADV_ID0,              /* Connect/Pair/Switch to Bluetooth Device 1  */
    ADV_ID1,              /* Connect/Pair/Switch to Bluetooth Device 2  */
    ADV_ID2,              /* Connect/Pair/Switch to Bluetooth Device 3  */
    ADV_ID3,              /* Connect/Pair/Switch to Bluetooth Device 4  */
    ADV_ID4,              /* Connect/Pair/Switch to Bluetooth Device 5  */
    DEL_ID0,              /* Delete bonding of Bluetooth Device 1       */
    DEL_ID1,              /* Delete bonding of Bluetooth Device 2       */
    DEL_ID2,              /* Delete bonding of Bluetooth Device 3       */
    DEL_ID3,              /* Delete bonding of Bluetooth Device 4       */
    DEL_ID4,              /* Delete bonding of Bluetooth Device 5       */
    ENT_DFU,              /* Turn on DFU mode                           */
    ENT_SLP,              /* Deep Sleep mode                            */
    BATT_LV,              /* Display battery level in milli volts(Beta) */
    QWERTY,
    LOWER,
    RAISE,
    ADJUST,
    BACKLIT,
    EISU,
    KANA,
    RGBRST,
    PLOVER,
    };


extern keymap_config_t keymap_config;

uint32_t layer_state_set_user(uint32_t state) {
  return update_tri_layer_state(state, _LOWER, _RAISE, _ADJUST);
}

// Optional - Fillers to make layering more clear
#define _______ KC_TRNS
#define XXXXXXX KC_NO
#define KC_XXXXX KC_NO
#define KC_LOWER LOWER
#define KC_RAISE RAISE
#define KC_RST   RESET
// Copy and paste from here

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

[0] = LAYOUT(
KC_GRV,      KC_CAPS,   KC_TAB,    KC_BSPC,   KC_LSFT,   KC_ESC,                   KC_BTN2,   KC_UP,   KC_DOWN,   KC_LEFT,   KC_RIGHT,  KC_EQL,  
LGUI(KC_A),  KC_1,      KC_2,      KC_3,      KC_4,      KC_5,                     KC_6,      KC_7,    KC_8,      KC_9,      KC_0,      KC_MINS, 
LGUI(KC_C),  KC_Q,      KC_W,      KC_E,      KC_R,      KC_T,                     KC_Y,      KC_U,    KC_I,      KC_O,      KC_P,      KC_LBRC, 
LGUI(KC_V),  KC_A,      KC_S,      KC_D,      KC_F,      KC_G,                     KC_H,      KC_J,    KC_K,      KC_L,      KC_SCLN,   KC_QUOT, 
LGUI(KC_Z),  KC_Z,      KC_X,      KC_C,      KC_V,      KC_B,                     KC_N,      KC_M,    KC_COMM,   KC_DOT,    KC_SLSH,   KC_BSLS, 
                                              KC_SPC,    KC_ENT,                   KC_LALT,   KC_LCTL, 
                                  LT(1, KC_BSPC), LT(2, KC_DEL),                   KC_LGUI,   KC_LSFT  
),

[1] = LAYOUT(
KC_TRNS,     KC_TRNS,   KC_TRNS,   KC_TRNS,   KC_TRNS,   KC_TRNS,                  KC_TRNS,   KC_TRNS,  KC_TRNS,  KC_TRNS,   KC_TRNS,   KC_TRNS, 
KC_TRNS,     KC_TRNS,   KC_TRNS,   KC_TRNS,   KC_TRNS,   KC_TRNS,                  KC_7,      KC_8,     KC_9,     KC_PSLS,   KC_TRNS,   KC_TRNS, 
KC_TRNS,     KC_TRNS,   KC_F1,     KC_F2,     KC_F3,     KC_F4,                    KC_4,      KC_5,     KC_6,     KC_PAST,   KC_TRNS,   KC_RBRC, 
KC_TRNS,     KC_TRNS,   KC_F5,     KC_F6,     KC_F7,     KC_F8,                    KC_1,      KC_2,     KC_3,     KC_MINS,   KC_TRNS,   KC_TRNS, 
KC_TRNS,     KC_TRNS,   KC_F9,     KC_F10,    KC_F11,    KC_F12,                   M(0),      KC_0,     KC_DOT,   KC_EQL,    KC_TRNS,   KC_TRNS, 
                                              KC_TRNS,   KC_TRNS,                  KC_TRNS,   KC_TRNS, 
                                              KC_TRNS,   KC_TRNS,                  KC_TRNS,   KC_TRNS  
),

[2] = LAYOUT(
KC_TRNS,     KC_TRNS,   KC_TRNS,   KC_TRNS,   KC_TRNS,   KC_TRNS,                  KC_TRNS,   KC_TRNS,  KC_TRNS,  KC_TRNS,   KC_TRNS,   KC_TRNS, 
KC_TRNS,     KC_TRNS,   KC_WH_L,   KC_WH_R,   KC_WH_U,   KC_WH_D,                  KC_INS,    KC_HOME,  KC_PGUP,  KC_VOLU,   KC_MPLY,   KC_TRNS, 
KC_TRNS,     KC_TRNS,   M(101),    M(102),    M(103),    M(104),                   KC_DEL,    KC_END,   KC_PGDN,  KC_VOLD,   KC_MSTP,   KC_TRNS, 
KC_TRNS,     KC_TRNS,   M(105),    M(106),    M(107),    M(108),                   KC_TRNS,   KC_UP,    KC_TRNS,  KC_MUTE,   KC_MPRV,   KC_TRNS, 
KC_TRNS,     KC_TRNS,   M(201),    M(202),    M(203),    M(204),                   KC_LEFT,   KC_DOWN,  KC_RIGHT, KC_TRNS,   KC_MNXT,   KC_TRNS, 
                                              KC_TRNS,    KC_TRNS,                 MO(4),    MO(5), 
                                              KC_TRNS,    KC_TRNS,                 MO(6),    MO(7) 
),

// Hold layer 1 & 2 for Adjust layer 
[3] = LAYOUT(
KC_TRNS,     KC_TRNS,   KC_TRNS,   KC_TRNS,   KC_TRNS,   KC_TRNS,                  KC_TRNS,   KC_TRNS,  KC_TRNS,  KC_TRNS,   KC_TRNS,   KC_TRNS, 
KC_TRNS,     DELBNDS,   KC_TRNS,   KC_TRNS,   USB_SEL,   BLE_SEL,                  KC_TRNS,   KC_TRNS,  KC_TRNS,  KC_TRNS,   KC_TRNS,   KC_TRNS, 
KC_TRNS,     ADV_ID0,   ADV_ID1,   ADV_ID2,   ADV_ID3,   ADV_ID4,                  KC_TRNS,   KC_TRNS,  KC_TRNS,  KC_TRNS,   KC_TRNS,   KC_TRNS, 
KC_TRNS,     DEL_ID0,   DEL_ID1,   DEL_ID2,   DEL_ID3,   DEL_ID4,                  KC_TRNS,   KC_TRNS,  KC_TRNS,  KC_TRNS,   KC_TRNS,   KC_TRNS, 
KC_TRNS,     ENT_DFU,   KC_TRNS,   KC_TRNS,   BATT_LV,   ENT_SLP,                  KC_TRNS,   KC_TRNS,  KC_TRNS,  KC_TRNS,   KC_TRNS,   KC_TRNS, 
                                              KC_TRNS,   KC_TRNS,                  KC_TRNS,   KC_TRNS, 
                                              KC_TRNS,   KC_TRNS,                  KC_TRNS,   KC_TRNS  
),

[4] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[5] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[6] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[7] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[8] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[9] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[10] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[11] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[12] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[13] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

[14] = LAYOUT(
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS, 
                                                KC_TRNS,    KC_TRNS,               KC_TRNS,    KC_TRNS  
),

};


const macro_t *action_get_macro(keyrecord_t *record, uint8_t id, uint8_t opt) {
 /* Macro Commands
A macro can include the following commands:
* I() change interval of stroke in milliseconds. MAX=255
* D() press key.
* U() release key.
* T() type key(press and release).
* W() wait (milliseconds). MAX=255
* END end mark. 
* MAX: You can set 255 MACROS. */

    switch (id) {

        case 101: // EXAMPLE 
        if (record->event.pressed) {    
            SEND_STRING("YOUR PHONE NUMNBER" SS_TAP(X_TAB)); 
          } 
            break;
        case 102: // EXAMPLE 
        if (record->event.pressed) {
            SEND_STRING("YOUR EMAIL" SS_TAP(X_ENTER));
          } 
            break;  
        case 103: // EXAMPLE 
        if (record->event.pressed) {
            SEND_STRING("WEBSITE ADDRESS");
          } 
            break;  
        case 104: // EXAMPLE 
        if (record->event.pressed) {
            SEND_STRING("YOUR NAME" SS_TAP(X_ENTER));
          } 
            break;  
        case 105: // EXAMPLE 
        if (record->event.pressed) {
            SEND_STRING("SEND STRING 5");
          } 
            break;  
        case 106: // EXAMPLE 
        if (record->event.pressed) {
            SEND_STRING("SEND STRING 6");
          } 
            break; 
        case 107: // EXAMPLE 
        if (record->event.pressed) {
            SEND_STRING("SEND STRING 7");
          } 
            break; 
        case 108: // EXAMPLE 
        if (record->event.pressed) {
            SEND_STRING("SEND STRING 8");
          } 
            break;  
        case 201: // EXAMPLE
        if (record->event.pressed) {
            return MACRO( T(1), T(2), T(ENT), END );
          } 
            break; 
        case 202: // EXAMPLE
        if (record->event.pressed) {
            return MACRO( T(3), T(4), T(ENT), END );
          } 
            break; 
        case 203:  // EXAMPLE
        if (record->event.pressed) {
            return MACRO( T(5), T(6), T(ENT), END );
          } 
            break; 
        case 204:  // EXAMPLE
        if (record->event.pressed) {
            return MACRO( T(7), T(8), T(ENT), END );
          } 
            break; 

        case 0: // FOR NUMPAD 00
            if (record->event.pressed) {
                return MACRO( T(0), T(0), END );
            }
            break;
    }
    return MACRO_NONE;
}


void persistent_default_layer_set(uint16_t default_layer) {
  eeconfig_update_default_layer(default_layer);
  default_layer_set(default_layer);
  layer_state_set(default_layer);
}

// Setting ADJUST layer RGB back to default
void update_tri_layer_RGB(uint8_t layer1, uint8_t layer2, uint8_t layer3) {
  if (IS_LAYER_ON(layer1) && IS_LAYER_ON(layer2)) {
    #ifdef RGBLIGHT_ENABLE
       //rgblight_mode(RGB_current_mode);
    #endif
    layer_on(layer3);
  } else {
    layer_off(layer3);
  }
}

// When add source files to SRC in rules.mk, you can use functions.
const char *read_layer_state(void);
const char *read_logo(void);
void set_keylog(uint16_t keycode, keyrecord_t *record);
const char *read_keylog(void);
const char *read_keylogs(void);


bool process_record_user(uint16_t keycode, keyrecord_t *record) {
  char str[16];

  //NRF_LOG_INFO("process_record_user, keycode: %d\n", keycode);

#ifdef SSD1306OLED
  iota_gfx_flush(); // wake up screen
#endif

  if (record->event.pressed) {
    set_keylog(keycode, record);
    //set_timelog();


    eeprom_write_dword(EECONFIG_RGBLIGHT, 666);
  }




  switch (keycode) {
    case RGB_TOG:
        if (record->event.pressed) {
            // only enable the power for now, never disable
            if (!rgblight_config.enable) {
                //nrfmicro_power_enable(!rgblight_config.enable);
            }
            #ifdef SSD1306OLED
                //iota_gfx_init(!IS_LEFT_HAND); // enable the OLED screen
            #endif
        }
    break;

    case QWERTY:
      if (record->event.pressed) {
        persistent_default_layer_set(1UL<<_QWERTY);
      }
      return false;
      break;

    case LOWER:
      if (record->event.pressed) {
          //not sure how to have keyboard check mode and set it to a variable, so my work around
          //uses another variable that would be set to true after the first time a reactive key is pressed.
        if (TOG_STATUS) { //TOG_STATUS checks is another reactive key currently pressed, only changes RGB mode if returns false
        } else {
          TOG_STATUS = !TOG_STATUS;
          #ifdef RGBLIGHT_ENABLE
            //rgblight_mode(16);
          #endif
        }
        layer_on(_LOWER);
        update_tri_layer_RGB(_LOWER, _RAISE, _ADJUST);
      } else {
        #ifdef RGBLIGHT_ENABLE
          //rgblight_mode(RGB_current_mode);   // revert RGB to initial mode prior to RGB mode change
        #endif
        TOG_STATUS = false;
        layer_off(_LOWER);
        update_tri_layer_RGB(_LOWER, _RAISE, _ADJUST);
      }
      return false;
      break;
    case RAISE:
      if (record->event.pressed) {
        //not sure how to have keyboard check mode and set it to a variable, so my work around
        //uses another variable that would be set to true after the first time a reactive key is pressed.
        if (TOG_STATUS) { //TOG_STATUS checks is another reactive key currently pressed, only changes RGB mode if returns false
        } else {
          TOG_STATUS = !TOG_STATUS;
          #ifdef RGBLIGHT_ENABLE
            //rgblight_mode(15);
          #endif
        }
        layer_on(_RAISE);
        update_tri_layer_RGB(_LOWER, _RAISE, _ADJUST);
      } else {
        #ifdef RGBLIGHT_ENABLE
          //rgblight_mode(RGB_current_mode);  // revert RGB to initial mode prior to RGB mode change
        #endif
        layer_off(_RAISE);
        TOG_STATUS = false;
        update_tri_layer_RGB(_LOWER, _RAISE, _ADJUST);
      }
      return false;
      break;
    case ADJUST:
        if (record->event.pressed) {
          layer_on(_ADJUST);
        } else {
          layer_off(_ADJUST);
        }
        return false;
        break;
      //led operations - RGB mode change now updates the RGB_current_mode to allow the right RGB mode to be set after reactive keys are released
    case RGB_MOD:
      #ifdef RGBLIGHT_ENABLE
        if (record->event.pressed) {
          rgblight_mode(RGB_current_mode);
          rgblight_step();
          RGB_current_mode = rgblight_config.mode;
          NRF_LOG_INFO("RGB_MOD, RGB_current_mode: %d\n", RGB_current_mode);
        }
      #endif
      return false;
      break;
    case EISU:
      if (record->event.pressed) {
        if(keymap_config.swap_lalt_lgui==false){
          register_code(KC_LANG2);
        }else{
          SEND_STRING(SS_LALT("`"));
        }
      } else {
        unregister_code(KC_LANG2);
      }
      return false;
      break;
    case KANA:
      if (record->event.pressed) {
        if(keymap_config.swap_lalt_lgui==false){
          register_code(KC_LANG1);
        }else{
          SEND_STRING(SS_LALT("`"));
        }
      } else {
        unregister_code(KC_LANG1);
      }
      return false;
      break;
    case RGBRST:
      #ifdef RGBLIGHT_ENABLE
        if (record->event.pressed) {
          nrfmicro_power_enable(true);
          eeconfig_update_rgblight_default();
          rgblight_enable();
          RGB_current_mode = rgblight_config.mode;
          NRF_LOG_INFO("RGBRST, RGB_current_mode: %d\n", RGB_current_mode);
        }
      #endif

      #ifdef SSD1306OLED
      //iota_gfx_init(!IS_LEFT_HAND); // enable the OLED screen
      #endif

      break;
  }
  if (record->event.pressed) {
    switch (keycode) {
    case DELBNDS:
      delete_bonds();
      return false;
      break;
    case AD_WO_L:
      restart_advertising_wo_whitelist();
      return false;
    case USB_EN:
      set_usb_enabled(true);
      return false;
      break;
    case USB_DIS:
      set_usb_enabled(false);
      return false;
      break;
    case BLE_EN:
      set_ble_enabled(true);
      return false;
      break;
    case BLE_DIS:
      set_ble_enabled(false);
      return false;
      break;
    case BLE_SEL:
      set_ble_enabled(true);
      set_usb_enabled(false);
      return false;
      break;
    case USB_SEL:
      set_ble_enabled(false);
      set_usb_enabled(true);
      return false;
      break;
    case ADV_ID0:
      restart_advertising_id(0);
      return false;
    case ADV_ID1:
      restart_advertising_id(1);
      return false;
    case ADV_ID2:
      restart_advertising_id(2);
      return false;
    case ADV_ID3:
      restart_advertising_id(3);
      return false;
    case ADV_ID4:
      restart_advertising_id(4);
      return false;
    case DEL_ID0:
      delete_bond_id(0);
      return false;
    case DEL_ID1:
      delete_bond_id(1);
      return false;
    case DEL_ID2:
      delete_bond_id(2);
      return false;
    case DEL_ID3:
      delete_bond_id(3);
      return false;
    case BATT_LV:
      sprintf(str, "%4dmV", get_vcc());
      send_string(str);
      return false;
    case ENT_DFU:
      bootloader_jump();
      return false;
    }
  }
  else if (!record->event.pressed) {
    switch (keycode) {
    case ENT_SLP:
      sleep_mode_enter();
      return false;
    }
  }

  return true;
}
;



#ifdef SSD1306OLED

void matrix_update(struct CharacterMatrix *dest,
                          const struct CharacterMatrix *source) {
  if (memcmp(dest->display, source->display, sizeof(dest->display))) {
    memcpy(dest->display, source->display, sizeof(dest->display));
    dest->dirty = true;
  }
}

//const char *read_host_led_state(void);
//const char *read_mode_icon(bool swap);

const char *read_rgb_info(void);

void matrix_render_user(struct CharacterMatrix *matrix) {


//  if (is_master) 
{
    //matrix_write_ln(matrix, read_layer_state());
    //matrix_write_ln(matrix, read_keylog());

    {
    char str[64];

#if (IS_LEFT_HAND)
      sprintf (str, "Master: %s%s%s",
        get_usb_enabled() && !get_ble_enabled() ? "USB mode":"",
        get_ble_enabled() && ble_connected() ? "connected":"",
        get_ble_enabled() && !ble_connected() ? "disconnected":""
      );
#else
      sprintf(str, "Slave: %s", ble_connected() ? "connected" : "disconnected");
#endif

      matrix_write_ln(matrix, str);
    }

    matrix_write_ln(matrix, read_rgb_info());

    {
      char vc[16], str[32];
      int vcc = get_vcc();
      sprintf(vc, "%4dmV", vcc);
      sprintf(str, "Bat: %s USB: %s", vcc==0 || vcc>4400 || nrfmicro_switch_pin()==0 ? "off   " : vc, has_usb()? "on":"off");
      //sprintf(str, "Switch pin: %d", nrfmicro_switch_pin());
      matrix_write_ln(matrix, str);
    }

    //matrix_write_ln(matrix, read_keylogs());
    //matrix_write_ln(matrix, read_mode_icon(keymap_config.swap_lalt_lgui));
    //matrix_write_ln(matrix, read_host_led_state());
    //matrix_write_ln(matrix, read_timelog());
  }
// else {
//    matrix_write_ln(matrix, read_layer_state()); // somehow removes the dead pixel
//    matrix_write(matrix, read_logo());
//  }

}

void iota_gfx_task_user(void) {
  ScreenOffInterval = has_usb() ? 60*10*1000 : 60*5*1000; // ms
  struct CharacterMatrix matrix;
  matrix_clear(&matrix);
  matrix_render_user(&matrix);
  matrix_update(&display, &matrix);
}

#endif
